import lensIcon from '@/assets/vectors/icons/lens.svg';
import { isDrawerOpenSelector } from '@/redux/drawer/drawer.selectors';
import { openSearchDrawerWithSelectedTab, selectTab } from '@/redux/drawer/drawer.slice';
import { useAppDispatch } from '@/redux/hooks/useAppDispatch';
import {
  isPendingSearchStatusSelector,
  perfectMatchSelector,
} from '@/redux/search/search.selectors';
import { getSearchData } from '@/redux/search/search.thunks';
import Image from 'next/image';
import { ChangeEvent, KeyboardEvent, useCallback, useEffect, useState } from 'react';
import { useRouter } from 'next/router';
import { useSelector } from 'react-redux';
import { getDefaultSearchTab, getSearchValuesArrayAndTrimToSeven } from './SearchBar.utils';

const ENTER_KEY_CODE = 'Enter';

export const SearchBar = (): JSX.Element => {
  const [searchValue, setSearchValue] = useState<string>('');
  const isPendingSearchStatus = useSelector(isPendingSearchStatusSelector);
  const isDrawerOpen = useSelector(isDrawerOpenSelector);
  const isPerfectMatch = useSelector(perfectMatchSelector);
  const dispatch = useAppDispatch();
  const router = useRouter();

  const updateSearchValueFromQueryParam = useCallback((): void => {
    const { searchValue: searchValueQueryParam } = router.query;
    if (typeof searchValueQueryParam === 'string') {
      setSearchValue(searchValueQueryParam);
    }
  }, [router.query]);

  const openSearchDrawerIfClosed = (defaultSearchTab: string): void => {
    if (!isDrawerOpen) {
      dispatch(openSearchDrawerWithSelectedTab(defaultSearchTab));
    } else {
      dispatch(selectTab(defaultSearchTab));
    }
  };

  const onSearchChange = (event: ChangeEvent<HTMLInputElement>): void => {
    setSearchValue(event.target.value);
  };

  const onSearchClick = (): void => {
    const searchValues = getSearchValuesArrayAndTrimToSeven(searchValue);

    dispatch(getSearchData({ searchQueries: searchValues, isPerfectMatch }));
    openSearchDrawerIfClosed(getDefaultSearchTab(searchValues));
  };

  const handleKeyPress = (event: KeyboardEvent<HTMLInputElement>): void => {
    const searchValues = getSearchValuesArrayAndTrimToSeven(searchValue);

    if (event.code === ENTER_KEY_CODE) {
      dispatch(getSearchData({ searchQueries: searchValues, isPerfectMatch }));
      openSearchDrawerIfClosed(getDefaultSearchTab(searchValues));
    }
  };

  useEffect(() => {
    updateSearchValueFromQueryParam();
  }, [updateSearchValueFromQueryParam]);

  return (
    <div className="relative" data-testid="search-bar">
      <input
        value={searchValue}
        name="search-input"
        aria-label="search-input"
        data-testid="search-input"
        onKeyDown={handleKeyPress}
        onChange={onSearchChange}
        disabled={isPendingSearchStatus}
        className="h-9 w-72 rounded-[64px] border border-transparent bg-cultured px-4 py-2.5 text-xs font-medium text-font-400 outline-none  hover:border-greyscale-600 focus:border-greyscale-600"
      />
      <button
        disabled={isPendingSearchStatus}
        type="button"
        className="bg-transparent"
        onClick={onSearchClick}
      >
        <Image
          src={lensIcon}
          alt="lens icon"
          height={16}
          width={16}
          className="absolute right-4 top-2.5"
        />
      </button>
    </div>
  );
};
