import { Icon } from '@/shared/Icon';
import { PinDetailsItem } from '@/types/models';
import { useAppSelector } from '@/redux/hooks/useAppSelector';
import { modelsDataSelector } from '@/redux/models/models.selectors';
import { SIZE_OF_EMPTY_ARRAY } from '@/constants/common';
import { useAppDispatch } from '@/redux/hooks/useAppDispatch';
import { openMapAndSetActive, setActiveMap } from '@/redux/map/map.slice';
import { mapModelIdSelector, mapOpenedMapsSelector } from '@/redux/map/map.selectors';
import { PluginsEventBus } from '@/services/pluginsManager/pluginsEventBus';
import { useSetBounds } from '@/utils/map/useSetBounds';
import { getListOfAvailableSubmaps, getPinColor } from './PinsListItem.component.utils';
import { AvailableSubmaps, PinTypeWithNone } from '../PinsList.types';
import { useVisiblePinsPolygonCoordinates } from './hooks/useVisiblePinsPolygonCoordinates';

interface PinsListItemProps {
  name: string;
  type: PinTypeWithNone;
  pin: PinDetailsItem;
}

export const PinsListItem = ({ name, type, pin }: PinsListItemProps): JSX.Element => {
  const dispatch = useAppDispatch();
  const openedMaps = useAppSelector(mapOpenedMapsSelector);
  const models = useAppSelector(modelsDataSelector);
  const availableSubmaps = getListOfAvailableSubmaps(pin, models);
  const currentModelId = useAppSelector(mapModelIdSelector);
  const coordinates = useVisiblePinsPolygonCoordinates(pin.targetElements);
  const setBounds = useSetBounds();

  const isMapAlreadyOpened = (modelId: number): boolean =>
    openedMaps.some(map => map.modelId === modelId);

  const onSubmapClick = (map: AvailableSubmaps): void => {
    if (isMapAlreadyOpened(map.modelId)) {
      dispatch(setActiveMap({ modelId: map.modelId }));
    } else {
      dispatch(openMapAndSetActive({ modelId: map.modelId, modelName: map.name }));
    }
    if (currentModelId !== map.modelId) {
      PluginsEventBus.dispatchEvent('onSubmapClose', currentModelId);
      PluginsEventBus.dispatchEvent('onSubmapOpen', map.modelId);
    }
  };

  const handleCenterMapToPin = (): void => {
    if (!coordinates) return;
    setBounds(coordinates);
  };

  return (
    <div className="mb-4 flex w-full flex-col gap-3 rounded-lg border-[1px] border-solid border-greyscale-500 p-4">
      <div className="flex w-full flex-row items-center gap-2">
        <button
          type="button"
          className="mr-2 shrink-0"
          onClick={handleCenterMapToPin}
          data-testid="center-to-pin"
        >
          <Icon name="pin" className={getPinColor(type)} />
        </button>
        <p>
          Full name: <span className="w-full font-bold">{name}</span>
        </p>
      </div>
      <ul className="leading-6">
        <div className="font-bold">Elements:</div>
        {'targetParticipants' in pin &&
          pin.targetParticipants.map(element => {
            return (
              <li key={element.id} className="my-2 px-2">
                <a
                  href={element.link}
                  target="_blank"
                  className="cursor-pointer text-primary-500 underline"
                >
                  {element.type} ({element.resource})
                </a>
              </li>
            );
          })}
      </ul>
      <ul className="leading-6">
        <div className="font-bold">References:</div>
        {pin.references.map(reference => {
          return (
            <li key={reference.id} className="my-2 px-2">
              <a
                href={reference.article?.link}
                target="_blank"
                className="cursor-pointer text-primary-500 underline"
              >
                {reference.type} ({reference.resource})
              </a>
            </li>
          );
        })}
      </ul>
      {availableSubmaps.length > SIZE_OF_EMPTY_ARRAY && (
        <ul className="leading-6">
          <div className="mb-2 font-bold">Available in submaps:</div>
          {availableSubmaps.map(submap => {
            return (
              <button
                onClick={(): void => onSubmapClick(submap)}
                className="mb-2 mr-2 rounded border border-solid border-greyscale-500 p-2 font-normal text-[#6A6977] hover:border-[#6A6977]"
                type="button"
                key={submap.id}
              >
                {submap.name}
              </button>
            );
          })}
        </ul>
      )}
    </div>
  );
};
