/* eslint-disable no-magic-numbers */
import { createdOverlayFixture } from '@/models/fixtures/overlaysFixture';
import { PluginsEventBus } from './pluginsEventBus';

describe('PluginsEventBus', () => {
  beforeEach(() => {
    PluginsEventBus.events = [];
  });
  it('should store event listener', () => {
    const callback = jest.fn();
    PluginsEventBus.addListener('123', 'onAddDataOverlay', callback);

    expect(PluginsEventBus.events).toEqual([
      {
        hash: '123',
        type: 'onAddDataOverlay',
        callback,
      },
    ]);
  });

  it('should dispatch event correctly', () => {
    const callback = jest.fn();
    PluginsEventBus.addListener('123', 'onAddDataOverlay', callback);
    PluginsEventBus.dispatchEvent('onAddDataOverlay', createdOverlayFixture);

    expect(callback).toHaveBeenCalled();
  });

  it('should throw error if event type is incorrect', () => {
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    expect(() => PluginsEventBus.dispatchEvent('onData' as any, createdOverlayFixture)).toThrow(
      'Invalid event type: onData',
    );
  });
  it('should remove listener only for specific plugin, event type, and callback', () => {
    const callback = (): void => {};

    PluginsEventBus.addListener('123', 'onAddDataOverlay', callback);
    PluginsEventBus.addListener('123', 'onBioEntityClick', callback);
    PluginsEventBus.addListener('234', 'onBioEntityClick', callback);

    expect(PluginsEventBus.events).toHaveLength(3);

    PluginsEventBus.removeListener('123', 'onAddDataOverlay', callback);
    expect(PluginsEventBus.events).toHaveLength(2);
    expect(PluginsEventBus.events).toEqual([
      {
        callback,
        hash: '123',
        type: 'onBioEntityClick',
      },
      {
        callback,
        hash: '234',
        type: 'onBioEntityClick',
      },
    ]);
  });
  it('should throw if listener is not defined by plugin', () => {
    const callback = (): void => {};

    PluginsEventBus.addListener('123', 'onAddDataOverlay', callback);
    PluginsEventBus.addListener('123', 'onBioEntityClick', callback);
    PluginsEventBus.addListener('234', 'onBioEntityClick', callback);

    expect(PluginsEventBus.events).toHaveLength(3);

    expect(() => PluginsEventBus.removeListener('123', 'onHideOverlay', callback)).toThrow(
      "Listener doesn't exist",
    );
    expect(PluginsEventBus.events).toHaveLength(3);
  });
  it('should not remove listener when event with the same event type is defined by the same plugin but with different callback', () => {
    const callback = (): void => {};
    const secondCallback = (): void => {};

    PluginsEventBus.addListener('123', 'onAddDataOverlay', callback);
    PluginsEventBus.addListener('123', 'onAddDataOverlay', secondCallback);

    PluginsEventBus.removeListener('123', 'onAddDataOverlay', callback);

    expect(PluginsEventBus.events).toHaveLength(1);
    expect(PluginsEventBus.events).toEqual([
      {
        callback: secondCallback,
        hash: '123',
        type: 'onAddDataOverlay',
      },
    ]);
  });
  it('should remove all listeners defined by specific plugin', () => {
    const callback = (): void => {};
    PluginsEventBus.addListener('123', 'onAddDataOverlay', callback);
    PluginsEventBus.addListener('123', 'onBackgroundOverlayChange', callback);
    PluginsEventBus.addListener('251', 'onSubmapOpen', callback);
    PluginsEventBus.addListener('123', 'onHideOverlay', callback);
    PluginsEventBus.addListener('123', 'onSubmapOpen', callback);
    PluginsEventBus.addListener('992', 'onSearch', callback);

    PluginsEventBus.removeAllListeners('123');

    expect(PluginsEventBus.events).toHaveLength(2);
    expect(PluginsEventBus.events).toEqual([
      {
        callback,
        hash: '251',
        type: 'onSubmapOpen',
      },
      {
        callback,
        hash: '992',
        type: 'onSearch',
      },
    ]);
  });
});
