/* eslint-disable no-magic-numbers */
import { ZERO } from '@/constants/common';
import { PLUGINS_CONTENT_ELEMENT_ATTR_NAME, PLUGINS_CONTENT_ELEMENT_ID } from '@/constants/plugins';
import { configurationFixture } from '@/models/fixtures/configurationFixture';
import { store } from '@/redux/store';
import { PluginsManager } from './pluginsManager';
import { configurationMapper } from './pluginsManager.utils';

jest.mock('../../redux/store');

const createWrapperInDocument = (): HTMLDivElement => {
  const wrapper = document.createElement('div');
  wrapper.id = PLUGINS_CONTENT_ELEMENT_ID;

  document.body.appendChild(wrapper);

  return wrapper;
};

const createPluginElement = (hash: string): void => {
  const element = document.createElement('div');
  element.setAttribute(PLUGINS_CONTENT_ELEMENT_ATTR_NAME, hash);

  const wrapper = document.querySelector(`#${PLUGINS_CONTENT_ELEMENT_ID}`);
  wrapper?.append(element);
};

describe('PluginsManager', () => {
  const originalWindow = { ...global.window };

  beforeEach(() => {
    global.window = { ...originalWindow };
  });

  afterEach(() => {
    global.window = originalWindow;
  });

  afterEach(() => {
    jest.clearAllMocks();
  });

  it('setHashedPlugin correctly computes hash and updates hashedPlugins', () => {
    const pluginUrl = 'https://example.com/plugin.js';
    const pluginScript = 'console.log("Hello, Plugin!");';

    PluginsManager.setHashedPlugin({ pluginUrl, pluginScript });

    expect(PluginsManager.hashedPlugins[pluginUrl]).toBe('edc7eeafccc9e1ab66f713298425947b');
  });

  it('init subscribes to store changes and updates minerva configuration', () => {
    (store.getState as jest.Mock).mockReturnValueOnce({
      configuration: { main: { data: configurationFixture } },
    });

    PluginsManager.init();

    expect(store.subscribe).toHaveBeenCalled();

    // Simulate store change
    (store.subscribe as jest.Mock).mock.calls[0][0]();

    expect(store.getState).toHaveBeenCalled();
    expect(window.minerva.configuration).toEqual(configurationMapper(configurationFixture));
  });
  it('init does not update minerva configuration when configuration is undefined', () => {
    (store.getState as jest.Mock).mockReturnValueOnce({
      configuration: { main: { data: undefined } },
    });

    PluginsManager.init();

    expect(store.subscribe).toHaveBeenCalled();

    // Simulate store change
    (store.subscribe as jest.Mock).mock.calls[0][0]();

    expect(store.getState).toHaveBeenCalled();
    expect(window.minerva.configuration).toBeUndefined();
  });

  it('registerPlugin dispatches action and returns element', () => {
    const pluginName = 'TestPlugin';
    const pluginVersion = '1.0.0';
    const pluginUrl = 'https://example.com/test-plugin.js';

    const result = PluginsManager.registerPlugin({ pluginName, pluginVersion, pluginUrl });

    expect(store.dispatch).toHaveBeenCalled();

    expect(result.element).toBeDefined();
  });

  it('createAndGetPluginContent creates a content in wrapper and returns element', () => {
    const pluginName = 'TestPlugin';
    const pluginVersion = '1.0.0';
    const pluginUrl = 'https://example.com/test-plugin.js';
    const hash = '128ce10ae1b46ec4bc6d7c07278b5c9e';
    PluginsManager.hashedPlugins = {
      [pluginUrl]: hash,
    };

    createWrapperInDocument();

    const result = PluginsManager.registerPlugin({ pluginName, pluginVersion, pluginUrl });

    expect(result.element).toBeDefined();
    expect(result.element.getAttribute(PLUGINS_CONTENT_ELEMENT_ATTR_NAME)).toBe(hash);

    expect(result.element.parentNode).toBeDefined();
    expect(result.element.parentElement?.id).toBe(PLUGINS_CONTENT_ELEMENT_ID);
  });

  it('removePluginContent removes content in wrapper and returns element', () => {
    const hash = '128ce10ae1b46ec4bc6d7c07278b5c9e';

    const wrapper = createWrapperInDocument();
    createPluginElement(hash);

    PluginsManager.removePluginContent({ hash });

    expect(
      document.querySelector(`div[${PLUGINS_CONTENT_ELEMENT_ATTR_NAME}="${hash}"]`),
    ).not.toBeInTheDocument();

    expect(wrapper?.childElementCount).toEqual(ZERO);
  });
});
